package in.slanglabs.assistants.retail;

import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class SlangRetailAssistantUtils {

    private static String TAG = "SlangRetailAssistantUtils";

    private static Map<String, Object> mConfigOptions;

    static void setConfigOptions(Map<String, Object> configOptions) {
        mConfigOptions = configOptions;
    }

    static Map<Locale, String> convertToLocaleMap(Map<Object, Object> stringMap) {
        Map<Locale, String> localeMap = new HashMap<>();
        for (Map.Entry<Object, Object> entry : stringMap.entrySet()) {
            localeMap.put(SlangRetailAssistantLocaleMap.getDefaultLocale((String) entry.getKey()), (String) entry.getValue());
        }
        return localeMap;
    }

    static Map<String, Object> convertToStringMap(Map<Object, Object> map) {
        Map<String, Object> stringMap = new HashMap<>();
        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            stringMap.put((String) entry.getKey(), entry.getValue());
        }
        return stringMap;
    }

    static String getStringConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof String ?
                            (String) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static Boolean getBooleanConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof Boolean ?
                            (boolean) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return false;
    }

    static Double getDoubleConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof Double ?
                            (Double) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static Integer getIntegerConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof Integer ?
                            (Integer) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static Map<Object, Object> getMapConfig(String key) {
        try {
            HashMap<Object, Object> map = new HashMap<>();
            if ((mConfigOptions != null) && mConfigOptions.containsKey(key) && (mConfigOptions.get(key) instanceof Map)) {
                map = (HashMap<Object, Object>) mConfigOptions.get(key);
                return map;
            }
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }
        return null;
    }

    static List<String> getStringListConfig(String key) {
        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return null;

        try {
            List<String> stringArray = (List<String>) mConfigOptions.get(key);
            if (null == stringArray || stringArray.size() == 0)
                return null;

            List<String> stringList = new ArrayList<>(stringArray.size());
            stringList.addAll(stringArray);
            return stringList;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static Item.Size getSizeConfig(String key) {
        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return null;

        try {
            HashMap<String, Object> map = (HashMap<String, Object>) mConfigOptions.get(key);
            Integer amount = (Integer) map.get("amount");
            Item.Size.Unit unit = Item.Size.Unit.valueOf((String) map.get("unit"));
            return new Item.Size(amount, unit);
        } catch (Exception exception) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }
        return null;
    }

    static Item.Quantity getQuantityConfig(String key) {
        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return null;

        try {
            HashMap<String, Object> map = (HashMap<String, Object>) mConfigOptions.get(key);
            Integer amount = (Integer) map.get("amount");
            Item.Quantity.Unit unit = Item.Quantity.Unit.valueOf((String) map.get("unit"));
            return new Item.Quantity(amount, unit);
        } catch (Exception exception) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }
        return null;
    }

    static RetailAssistantUIPosition getAssistantUIConfig(String key) {
        RetailAssistantUIPosition.UIPosition uiPosition
                = RetailAssistantUIPosition.UIPosition.BOTTOM_CENTER;
        RetailAssistantUIPosition retailAssistantUIPosition
                = new RetailAssistantUIPosition.Builder()
                .setBaseUIPosition(uiPosition)
                .setIsDraggable(false)
                .setIsForcedAtStartup(false)
                .build();

        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return retailAssistantUIPosition;

        try {
            HashMap<String, Object> assistantUIMap = (HashMap<String, Object>) mConfigOptions.get(key);
            if (null == assistantUIMap)
                return retailAssistantUIPosition;
            String baseUIPosition = (String) assistantUIMap.get("baseUIPosition");
            switch (baseUIPosition != null ? baseUIPosition : "") {
                case "BOTTOM_LEFT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.BOTTOM_LEFT;
                    break;
                case "CENTER_LEFT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.CENTER_LEFT;
                    break;
                case "CENTER":
                    uiPosition = RetailAssistantUIPosition.UIPosition.CENTER;
                    break;
                case "CENTER_RIGHT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.CENTER_RIGHT;
                    break;
                case "TOP_LEFT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.TOP_LEFT;
                    break;
                case "TOP_CENTER":
                    uiPosition = RetailAssistantUIPosition.UIPosition.TOP_CENTER;
                    break;
                case "TOP_RIGHT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.TOP_RIGHT;
                    break;
                case "BOTTOM_RIGHT":
                    uiPosition = RetailAssistantUIPosition.UIPosition.BOTTOM_RIGHT;
                    break;
                default:
                    uiPosition = RetailAssistantUIPosition.UIPosition.BOTTOM_CENTER;
            }
            int offsetX = assistantUIMap.containsKey("offsetX") ? (int) assistantUIMap.get("offsetX") : -1;
            int offsetY = assistantUIMap.containsKey("offsetY") ? (int) assistantUIMap.get("offsetY") : -1;
            boolean draggable = assistantUIMap.containsKey("draggable") && (boolean) assistantUIMap.get("draggable");
            boolean forcedAtStartup = assistantUIMap.containsKey("forcedAtStartup") && (boolean) assistantUIMap.get("forcedAtStartup");
            retailAssistantUIPosition = new RetailAssistantUIPosition.Builder()
                    .setBaseUIPosition(uiPosition)
                    .setOffsetXDp(offsetX)
                    .setOffsetYDp(offsetY)
                    .setIsDraggable(draggable)
                    .setIsForcedAtStartup(forcedAtStartup)
                    .build();
            return retailAssistantUIPosition;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
            return retailAssistantUIPosition;
        }
    }

    static Map<String, Object> mapSearchInfo(SearchInfo searchInfo) {
        if (searchInfo == null) {
            return null;
        }
        Map<String, Object> searchInfoMap = new HashMap<>();
        if (searchInfo != null) {
            searchInfoMap.put("item", mapItem(searchInfo.getItem()));
            searchInfoMap.put("filterInfoList", mapFilterInfoList(searchInfo.getFilters()));
            searchInfoMap.put("sortingInfo", mapSortingInfo(searchInfo.getSorting()));
            searchInfoMap.put("isAddToCart", searchInfo.isAddToCart());
        }
        return searchInfoMap;
    }

    static Map<String, Object> mapItem(Item item) {
        if (item == null) {
            return null;
        }
        Map<String, Object> itemMap = new HashMap<>();
        itemMap.put("id", item.getId());
        itemMap.put("brand", item.getBrand());
        itemMap.put("productType", item.getProductType());
        itemMap.put("description", item.getDescription());
        itemMap.put("category", item.getCategory());
        itemMap.put("variants", mapItemVariantList(item.getVariants()));
        itemMap.put("quantity", mapItemQuantity(item.getQuantity()));
        itemMap.put("size", mapItemSize(item.getSize()));
        itemMap.put("price", mapItemPrice(item.getPrice()));
        return itemMap;
    }

    static ArrayList<Object> mapFilterInfoList(List<FilterInfo> filterInfos) {
        ArrayList<Object> filterInfoListArray = new ArrayList<>();
        if (filterInfos != null) {
            for (FilterInfo filterInfo : filterInfos) {
                filterInfoListArray.add(mapFilterInfo(filterInfo));
            }
        }
        return filterInfoListArray;
    }

    static Map<String, Object> mapNavigationInfo(NavigationInfo navigationInfo) {
        if (navigationInfo == null) {
            return null;
        }
        Map<String, Object> navigationInfoMap = new HashMap<>();
        navigationInfoMap.put("target", navigationInfo.getTarget());
        return navigationInfoMap;
    }

    static Map<String, Object> mapOrderInfo(OrderInfo orderInfo) {
        if (orderInfo == null) {
            return null;
        }
        Map<String, Object> orderInfoMap = new HashMap<>();
        orderInfoMap.put("index", orderInfo.getIndex());
        orderInfoMap.put("action", orderInfo.getAction().toString());
        orderInfoMap.put("cancelConfirmationStatus", orderInfo.getCancelConfirmationStatus().toString());
        return orderInfoMap;
    }

    static Map<String, Object> mapOfferInfo(OfferInfo offerInfo) {
        if (offerInfo == null) {
            return null;
        }
        Map<String, Object> offerInfoMap = new HashMap<>();
        offerInfoMap.put("offerAction", offerInfo.getOfferAction().name());
        offerInfoMap.put("offerItem", mapItem(offerInfo.getOfferItem()));
        return offerInfoMap;
    }

    static Map<String, Object> mapCheckoutInfo(CheckoutInfo checkoutInfo) {
        if (checkoutInfo == null) {
            return null;
        }
        Map<String, Object> checkoutInfoMap = new HashMap<>();
        checkoutInfoMap.put("checkoutName", checkoutInfo.getCheckoutName());
        checkoutInfoMap.put("checkoutDeliveryDate", checkoutInfo.getCheckoutDeliveryDate());
        checkoutInfoMap.put("checkoutPhoneNumber", checkoutInfo.getCheckoutPhoneNumber());
        return checkoutInfoMap;
    }

    private static Map<String, Object> mapSortingInfo(SortingInfo sortingInfo) {
        if (sortingInfo == null) {
            return null;
        }
        Map<String, Object> sortingInfoMap = new HashMap<>();
        if (sortingInfo != null) {
            sortingInfoMap.put("sortOrder", sortingInfo.getSortOrder().toString());
            sortingInfoMap.put("sortType", sortingInfo.getSortType().toString());
        }
        return sortingInfoMap;
    }

    static ArrayList<Object> mapItemVariantList(String[] variants) {
        ArrayList<Object> variantsListArray = new ArrayList<>();
        if (variants != null) {
            for (String variant : variants) {
                variantsListArray.add(variant);
            }
        }
        return variantsListArray;
    }

    static Map<String, Object> mapItemQuantity(Item.Quantity itemQuantity) {
        if (itemQuantity == null) {
            return null;
        }
        Map<String, Object> itemQuantityMap = new HashMap<>();
        if (itemQuantity != null) {
            itemQuantityMap.put("amount", itemQuantity.getAmount());
            itemQuantityMap.put("unit", itemQuantity.getUnit().toString());
        }
        return itemQuantityMap;
    }

    static Map<String, Object> mapItemSize(Item.Size itemSize) {
        if (itemSize == null) {
            return null;
        }
        Map<String, Object> itemSizeMap = new HashMap<>();
        if (itemSize != null) {
            itemSizeMap.put("amount", itemSize.getAmount());
            itemSizeMap.put("unit", itemSize.getUnit().toString());
        }
        return itemSizeMap;
    }

    static Map<String, Object> mapItemPrice(Item.Price itemPrice) {
        if (itemPrice == null) {
            return null;
        }
        Map<String, Object> itemPriceMap = new HashMap<>();
        if (itemPrice != null) {
            itemPriceMap.put("minAmount", itemPrice.getMinAmount());
            itemPriceMap.put("maxAmount", itemPrice.getMaxAmount());
            itemPriceMap.put("currency", itemPrice.getCurrency().toString());
        }
        return itemPriceMap;
    }

    private static Map<String, Object> mapFilterInfo(FilterInfo filterInfo) {
        if (filterInfo == null) {
            return null;
        }
        Map<String, Object> filterInfoMap = new HashMap<>();
        if (filterInfo != null) {
            filterInfoMap.put("filterName", filterInfo.getFilterName());
            filterInfoMap.put("filterValue", filterInfo.getFilterValue());
        }
        return filterInfoMap;
    }

}
