import 'package:flutter/material.dart';
import 'dart:async';

import 'package:slang_retail_assistant/slang_retail_assistant.dart';
import 'dart:convert';

void main() {
  runApp(new MaterialApp(
    home: new MyApp(),
    debugShowCheckedModeBanner: false,
  ));
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp>
    implements RetailAssistantAction, RetailAssistantLifeCycleObserver {
  String _searchText = '';
  SearchUserJourney _searchUserJourney;
  NavigationUserJourney _navigationUserJourney;
  OrderManagementUserJourney _orderManagementUserJourney;
  SearchAppState searchAppState;
  NavigationAppState navigationAppState;
  OrderManagementAppState orderManagementAppState;
  int mOrderIndex = -1;

  @override
  void initState() {
    super.initState();
    initSlangRetailAssistant();
  }

  void initSlangRetailAssistant() {
    var positionConfig = new AssistantUIPosition()
      ..baseUIPosition = BaseUIPosition.BOTTOM_CENTER
      ..offsetY = -56
      ..isForced = true
      ..isDraggable = true;

    var assistantConfig = new AssistantConfiguration()
      ..assistantId = "AssistantId"
      ..apiKey = "AssistantAPIKey"
      ..environment = AssistantEnvironment.STAGING
      ..uiPosition = positionConfig;

    SlangRetailAssistant.initialize(assistantConfig);
    SlangRetailAssistant.setAction(this);
    SlangRetailAssistant.setLifecycleObserver(this);

    SearchUserJourney.getContext().clear();
    SearchUserJourney.disablePreserveContext();
    OrderManagementUserJourney.getContext().clear();
    OrderManagementUserJourney.disablePreserveContext();
    NavigationUserJourney.getContext().clear();
    NavigationUserJourney.disablePreserveContext();
  }

  @override
  void onAssistantError(Map<String, String> assistantError) {
    print("AssistantError main.dart" + assistantError.toString());
  }

  @override
  SearchAppState onSearch(
      SearchInfo searchInfo, SearchUserJourney searchUserJourney) {
    _searchUserJourney = searchUserJourney;
    setState(() {
      try {
        JsonEncoder encoder = new JsonEncoder.withIndent('  ');
        String searchMapString = encoder.convert(searchInfo);
        _searchText = searchMapString;
      } catch (e) {
        print(e);
      }
    });
    _showAppStateDialogForSearch().then((value) => (searchAppState !=
            SearchAppState.UNSUPPORTED)
        ? _showAppStateConditionDialogForSearch()
            .then((value) => _searchUserJourney.notifyAppState(searchAppState))
        : _searchUserJourney.notifyAppState(searchAppState));
    return SearchAppState.WAITING;
  }

  @override
  NavigationAppState onNavigation(
      navigationInfo, NavigationUserJourney navigationUserJourney) {
    _navigationUserJourney = navigationUserJourney;

    _showAppStateDialogForNavigation().then((value) =>
        navigationAppState != NavigationAppState.UNSUPPORTED
            ? _showAppStateConditionDialogForNavigation().then((value) =>
                _navigationUserJourney.notifyAppState(navigationAppState))
            : _navigationUserJourney.notifyAppState(navigationAppState));
    return NavigationAppState.WAITING;
  }

  @override
  OrderManagementAppState onOrderManagement(OrderInfo orderInfo,
      OrderManagementUserJourney orderManagementUserJourney) {
    _orderManagementUserJourney = orderManagementUserJourney;
    if (orderInfo.index != null) {
      setState(() {
        mOrderIndex = orderInfo.index;
      });
    }
    _showAppStateDialogForOrderManagement().then((value) =>
        (orderManagementAppState != OrderManagementAppState.UNSUPPORTED)
            ? _showAppStateConditionDialogForOrderManagement().then((value) =>
                _orderManagementUserJourney
                    .notifyAppState(orderManagementAppState))
            : _orderManagementUserJourney
                .notifyAppState(orderManagementAppState));
    return OrderManagementAppState.WAITING;
  }

  @override
  Widget build(BuildContext context) {
    SlangRetailAssistant.getUI().showTrigger();
    return MaterialApp(
        debugShowCheckedModeBanner: false,
        home: Scaffold(
            appBar: AppBar(
              title: const Text('Slang Retail PlayGround App'),
            ),
            body: Center(
                child: Column(
              mainAxisSize: MainAxisSize.max,
              children: [
                Container(height: 16), // set height
                FlatButton(
                  minWidth: 50.0,
                  height: 50.0,
                  child: Text(
                    'Show Trigger',
                    style: TextStyle(fontSize: 20.0),
                  ),
                  color: Colors.blueAccent,
                  textColor: Colors.white,
                  onPressed: () {
                    SlangRetailAssistant.getUI().showTrigger();
                  },
                ),
                Container(height: 16), // set height
                FlatButton(
                  minWidth: 50.0,
                  height: 50.0,
                  child: Text(
                    'Hide Trigger',
                    style: TextStyle(fontSize: 20.0),
                  ),
                  color: Colors.blueAccent,
                  textColor: Colors.white,
                  onPressed: () {
                    SlangRetailAssistant.getUI().hideTrigger();
                  },
                ),
                Container(height: 16), // set height
                FlatButton(
                  minWidth: 50.0,
                  height: 50.0,
                  child: Text(
                    'Clear UJ Context',
                    style: TextStyle(fontSize: 20.0),
                  ),
                  color: Colors.blueAccent,
                  textColor: Colors.white,
                  onPressed: () {
                    SearchUserJourney.getContext().clear();
                    OrderManagementUserJourney.getContext().clear();
                    NavigationUserJourney.getContext().clear();
                  },
                ),
                Container(height: 16), // set height
                FlatButton(
                  minWidth: 50.0,
                  height: 50.0,
                  child: Text(
                    'Notify Non-Voice Search',
                    style: TextStyle(fontSize: 20.0),
                  ),
                  color: Colors.blueAccent,
                  textColor: Colors.white,
                  onPressed: () {
                    SlangRetailAssistant.notifyNonVoiceSearch("potato");
                  },
                ),
                Container(height: 16), // set height
                Flexible(
                    child: FractionallySizedBox(
                        widthFactor: 0.9,
                        heightFactor: 0.98,
                        child: SingleChildScrollView(
                            physics: const AlwaysScrollableScrollPhysics(),
                            child: Container(
                              height: MediaQuery.of(context).size.height,
                              decoration: new BoxDecoration(
                                shape: BoxShape.rectangle,
                                color: Colors.black,
                              ),
                              child: Padding(
                                padding: const EdgeInsets.all(16.0),
                                child: Text(
                                  '$_searchText\n',
                                  style: TextStyle(
                                      fontSize: 20.0,
                                      fontWeight: FontWeight.bold,
                                      color: Colors.white),
                                ),
                              ),
                            ))))
              ],
            ))));
  }

  Future<void> _showAppStateDialogForSearch() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('Search App State'),
          content: SingleChildScrollView(
            child: ListBody(
              children: <Widget>[
                Text('Please select the search App state'),
              ],
            ),
          ),
          actions: <Widget>[
            TextButton(
              child: Text('ADD_TO_CART'),
              onPressed: () {
                setState(() {
                  searchAppState = SearchAppState.ADD_TO_CART;
                });
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('SEARCH_RESULTS'),
              onPressed: () {
                setState(() {
                  searchAppState = SearchAppState.SEARCH_RESULTS;
                });
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('UNSUPPORTED'),
              onPressed: () {
                setState(() {
                  searchAppState = SearchAppState.UNSUPPORTED;
                });
                Navigator.of(context).pop();
              },
            )
          ],
        );
      },
    );
  }

  Future<void> _showAppStateConditionDialogForSearch() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('Search App State'),
          content: Column(
            children: <Widget>[..._widgetListForSearchAppStateCondition],
          ),
        );
      },
    );
  }

  List<Widget> get _widgetListForSearchAppStateCondition {
    List<Widget> list = [];
    list.addAll([
      TextButton(
        child: Text('Success'),
        onPressed: () {
          _searchUserJourney.setSuccess();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Failure'),
        onPressed: () {
          _searchUserJourney.setFailure();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Item Not Found'),
        onPressed: () {
          _searchUserJourney.setItemNotFound();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Item Not Specified'),
        onPressed: () {
          _searchUserJourney.setItemNotSpecified();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Item Out of Stock'),
        onPressed: () {
          _searchUserJourney.setItemOutOfStock();
          Navigator.of(context).pop();
        },
      )
    ]);
    if (searchAppState != null &&
        searchAppState == SearchAppState.ADD_TO_CART) {
      list.addAll([
        TextButton(
          child: Text('Item Ambiguous'),
          onPressed: () {
            _searchUserJourney.setNeedDisambiguation();
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text('Item Ambiguous Force UI'),
          onPressed: () {
            _searchUserJourney.setStopDisambiguation();
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text('Item Quantity Required'),
          onPressed: () {
            _searchUserJourney.setNeedItemQuantity();
            Navigator.of(context).pop();
          },
        )
      ]);
    }
    return list;
  }

  Future<void> _showAppStateDialogForNavigation() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('Navigation App State'),
          content: SingleChildScrollView(
            child: ListBody(
              children: <Widget>[
                Text('Please select the Navigation App state'),
              ],
            ),
          ),
          actions: <Widget>[
            TextButton(
              child: Text('NAVIGATION'),
              onPressed: () {
                setState(() {
                  navigationAppState = NavigationAppState.NAVIGATION;
                });
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('UNSUPPORTED'),
              onPressed: () {
                setState(() {
                  navigationAppState = NavigationAppState.UNSUPPORTED;
                });
                Navigator.of(context).pop();
              },
            )
          ],
        );
      },
    );
  }

  Future<void> _showAppStateConditionDialogForNavigation() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('Navigation AppState Condition'),
          actions: [
            TextButton(
              child: Text('Success'),
              onPressed: () {
                _navigationUserJourney.setSuccess();
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Failure'),
              onPressed: () {
                _navigationUserJourney.setFailure();
                Navigator.of(context).pop();
              },
            ),
          ],
        );
      },
    );
  }

  Future<void> _showAppStateDialogForOrderManagement() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('OrderManagement App State'),
          content: SingleChildScrollView(
            child: ListBody(
              children: <Widget>[
                Text('Please select the OrderManagement App state'),
              ],
            ),
          ),
          actions: <Widget>[
            TextButton(
              child: Text('VIEW_ORDER'),
              onPressed: () {
                setState(() {
                  orderManagementAppState = OrderManagementAppState.VIEW_ORDER;
                });
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('CANCEL_ORDER'),
              onPressed: () {
                setState(() {
                  orderManagementAppState =
                      OrderManagementAppState.CANCEL_ORDER;
                });
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('UNSUPPORTED'),
              onPressed: () {
                setState(() {
                  orderManagementAppState = OrderManagementAppState.UNSUPPORTED;
                });
                Navigator.of(context).pop();
              },
            )
          ],
        );
      },
    );
  }

  Future<void> _showAppStateConditionDialogForOrderManagement() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('OrderManagement App State'),
          content: SingleChildScrollView(
              child: ListBody(
            children: <Widget>[
              ..._widgetListForOrderManagementAppStateCondition
            ],
          )),
        );
      },
    );
  }

  List<Widget> get _widgetListForOrderManagementAppStateCondition {
    List<Widget> list = [];
    list.addAll([
      TextButton(
        child: Text('Success'),
        onPressed: () {
          _orderManagementUserJourney.setSuccess();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Failure'),
        onPressed: () {
          _orderManagementUserJourney.setFailure();
          Navigator.of(context).pop();
        },
      ),
      TextButton(
        child: Text('Order Not Found'),
        onPressed: () {
          _orderManagementUserJourney.setOrderNotFound(-1);
          Navigator.of(context).pop();
        },
      )
    ]);
    if (orderManagementAppState != null &&
        orderManagementAppState == OrderManagementAppState.VIEW_ORDER) {
      list.addAll([
        TextButton(
          child: Text('Empty Order'),
          onPressed: () {
            _orderManagementUserJourney.setOrdersEmpty();
            Navigator.of(context).pop();
          },
        )
      ]);
    }
    if (orderManagementAppState != null &&
        orderManagementAppState == OrderManagementAppState.CANCEL_ORDER) {
      list.addAll([
        TextButton(
          child: Text('Cancel Confirmation Required'),
          onPressed: () {
            _orderManagementUserJourney.setConfirmationRequired();
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text('User Confirmed Cancel'),
          onPressed: () {
            _orderManagementUserJourney.setUserConfirmedCancel();
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text('User Denied Cancel'),
          onPressed: () {
            _orderManagementUserJourney.setUserDeniedCancel();
            Navigator.of(context).pop();
          },
        ),
      ]);
    }
    return list;
  }

  Future<void> _showMyDialogForOrderManagement() async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          elevation: 24.0,
          title: Text('OrderManagement App State Condition'),
          content: SingleChildScrollView(
            child: ListBody(
              children: <Widget>[
                Text('Please select the orderManagement App state condition'),
              ],
            ),
          ),
          actions: <Widget>[
            TextButton(
              child: Text('View Order Success'),
              onPressed: () {
                _orderManagementUserJourney.setSuccess();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.VIEW_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('View Order Empty'),
              onPressed: () {
                _orderManagementUserJourney.setOrdersEmpty();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.VIEW_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('View Order Not found'),
              onPressed: () {
                _orderManagementUserJourney.setOrderNotFound(mOrderIndex);
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.VIEW_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('View Order Failure'),
              onPressed: () {
                _orderManagementUserJourney.setFailure();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.VIEW_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order Success'),
              onPressed: () {
                _orderManagementUserJourney.setSuccess();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order Confirmation Required'),
              onPressed: () {
                _orderManagementUserJourney.setConfirmationRequired();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order User Confirm Cancel'),
              onPressed: () {
                _orderManagementUserJourney.setUserConfirmedCancel();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order User Denied Cancel'),
              onPressed: () {
                _orderManagementUserJourney.setUserDeniedCancel();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order Not Found'),
              onPressed: () {
                _orderManagementUserJourney.setOrderNotFound(mOrderIndex);
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              child: Text('Cancel Order Failure'),
              onPressed: () {
                _orderManagementUserJourney.setFailure();
                _orderManagementUserJourney
                    .notifyAppState(OrderManagementAppState.CANCEL_ORDER);
                Navigator.of(context).pop();
              },
            ),
          ],
        );
      },
    );
  }

  @override
  void onAssistantClosed(bool isCancelled) {
    print("onAssistantClosed " + isCancelled.toString());
  }

  @override
  void onAssistantInitFailure(String description) {
    print("onAssistantInitFailure " + description);
  }

  @override
  void onAssistantInitSuccess() {
    print("onAssistantInitSuccess");
  }

  @override
  void onAssistantInvoked() {
    print("onAssistantInvoked");
  }

  @override
  void onAssistantLocaleChanged(Map<String, String> locale) {
    print("onAssistantLocaleChanged " + locale.toString());
  }

  @override
  void onOnboardingFailure() {
    print("onOnboardingFailure");
  }

  @override
  void onOnboardingSuccess() {
    print("onOnboardingSuccess");
  }

  @override
  void onUnrecognisedUtterance(String utterance) {
    print("onUnrecognisedUtterance " + utterance);
  }

  @override
  void onUtteranceDetected(String utterance) {
    print("onUtteranceDetected " + utterance);
  }

  @override
  void onMicPermissionDenied() {
    print("onMicPermissionDenied");
  }

  @override
  void onMicPermissionGranted() {
    print("onMicPermissionDenied");
  }

  @override
  void onCoachmarkAction(AssistantCoachmarkType coachmarkType,
      AssistantCoachmarkAction coachmarkAction) {
    print(
        "onCoachmarkAction CoachmarkType=$coachmarkType CoachmarkAction=$coachmarkAction");
  }
}
