part of slang_retail_assistant;

class SearchUserJourney {
  SearchUserJourney._();

  static const MethodChannel _channel =
      const MethodChannel('plugins.flutter.io/slang_retail_assistant');

  static SearchContext getContext() {
    SearchContext._channel = _channel;
    return const SearchContext();
  }

  static void disablePreserveContext() {
    _channel.invokeMethod("disableSearchUserJourneyContext");
  }

  void setSuccess() {
    _channel.invokeMethod("setSearchSuccess");
  }

  void setFailure() {
    _channel.invokeMethod("setSearchFailure");
  }

  void setItemNotFound() {
    _channel.invokeMethod("setSearchItemNotFound");
  }

  void setItemOutOfStock() {
    _channel.invokeMethod("setSearchItemOutOfStock");
  }

  void setItemNotSpecified() {
    _channel.invokeMethod("setSearchItemNotSpecified");
  }

  void setNeedItemQuantity() {
    _channel.invokeMethod("setSearchNeedItemQuantity");
  }

  void setNeedDisambiguation() {
    _channel.invokeMethod("setSearchNeedDisambiguation");
  }

  void setStopDisambiguation() {
    _channel.invokeMethod("setSearchStopDisambiguation");
  }

  void setUserJourneyToContinue(
      AssistantUserJourney assistantUserJourney, bool isSpoken) {
    _channel.invokeMapMethod("setUserJourneyToContinue", {
      "currentUserJourney": AssistantUserJourney.SEARCH.stringValue,
      "userJourneyToContinue": assistantUserJourney.stringValue,
      "isSpoken": isSpoken
    });
  }

  void clearUserJourneyToContinue() {
    _channel.invokeMapMethod("clearUserJourneyToContinue",
        {"currentUserJourney": AssistantUserJourney.SEARCH.stringValue});
  }

  void notifyAppState(SearchAppState appState) {
    if (!SlangRetailAssistant._validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "notifySearchActionCompleted", {"appState": appState.stringValue});
  }
}

class SearchContext {
  const SearchContext();

  static late MethodChannel _channel;

  void clear() {
    _channel.invokeMethod("clearSearchUserJourneyContext");
  }

  void setItemBrand(brand) {
    _channel.invokeMethod("setItemBrand", {"brand": brand});
  }

  void setItemCategory(category) {
    _channel.invokeMethod("setItemCategory", {"category": category});
  }

  void setItemProductType(productType) {
    _channel.invokeMethod("setItemDescription", {"productType": productType});
  }

  void setItemDescription(description) {
    _channel.invokeMethod("setItemDescription", {"description": description});
  }

  void setItemPrice(Price price) {
    _channel.invokeMethod("setItemDescription", {"price": price.toJson()});
  }

  void setItemSize(Size size) {
    _channel.invokeMethod("setItemSize", {"size": size.toJson()});
  }

  void setItemVariants(List<String> variants) {
    _channel.invokeMethod("setItemVariants", {"variants": variants});
  }

  void setItemQuantity(Quantity quantity) {
    _channel.invokeMethod("setItemQuantity", {"quantity": quantity});
  }
}

enum SearchAppState { UNSUPPORTED, WAITING, SEARCH_RESULTS, ADD_TO_CART }

extension SearchAppStateExtension on SearchAppState {
  String get stringValue {
    switch (this) {
      case SearchAppState.UNSUPPORTED:
        return 'unsupported';
      case SearchAppState.WAITING:
        return 'waiting';
      case SearchAppState.SEARCH_RESULTS:
        return 'search_results';
      case SearchAppState.ADD_TO_CART:
        return 'add_to_cart';
      default:
        return "unsupported";
    }
  }
}
